/*
 * Worldcup is Copyright (c) 2001,2002 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "worldcup.h"
#include "worldcuprcp.h"

DmOpenRef worldcupDB, worldcupPics, datebook;
struct prefs thePrefs;

Boolean OpenDatebook()
{
  LocalID id = 0;
  DmSearchStateType state; 
  UInt16 card;
  Err theErr;

  /*
   * Try to open the datebook database
   */
  datebook = 0;
  theErr = DmGetNextDatabaseByTypeCreator(true, &state, 'DATA', 
					  'date', true, &card, &id); 
  if ( (theErr != dmErrCantFind) ) {
    datebook = DmOpenDatabase(card, id, dmModeReadWrite);
    return true;
  }
  return false;
}

Boolean OpenDatabase()
{
  LocalID id = 0;
  DmSearchStateType state; 
  UInt16 card;
  Err theErr;
  MemHandle handle;
  char *p;

  /*
   * Open the main database; it includes the schedule, team names etc.
   */
  theErr = DmGetNextDatabaseByTypeCreator(true, &state, 'Data', 
					  'tpWC', true, &card, &id); 

  if ( (theErr == dmErrCantFind) || 
       (0 == (worldcupDB = DmOpenDatabase(card, id, dmModeReadWrite)) ) ) {
    DisplayErrorMessage(IdErrorMsg);
    return false;
  }

  if (! (handle = DmQueryRecord(worldcupDB, 0)) ) {
    DisplayErrorMessage(IdErrorMsg+1);
    DmCloseDatabase(worldcupDB);
    return false;
  }

  p = MemHandleLock(handle);
  MemMove(&thePrefs, p, sizeof(thePrefs));
  MemHandleUnlock(handle);
  
  if (thePrefs.version != 1) {
    DisplayErrorMessage(IdErrorMsg+2);
    DmCloseDatabase(worldcupDB);
    return false;
  }

  /*
   * Try to open the optional Pictures database
   */
  theErr = DmGetNextDatabaseByTypeCreator(true, &state, 'Pics', 
					  'tpWC', true, &card, &id); 
  if ( (theErr != dmErrCantFind) )
    worldcupPics = DmOpenDatabase(card, id, dmModeReadOnly);
  
  return true;
}

char* ReadFixture(char* pos, struct spiel* spiel)
{
  MemHandle handle;
  static char *start;
  static UInt32 size;
  if (!pos) {
    handle = DmQueryRecord(worldcupDB,1);
    start = pos = MemHandleLock(handle);
    size = MemHandleSize(handle);
  } else {
    pos += sizeof(struct spiel);
  }
  if (pos-start>size) {
    MemPtrUnlock(start);
    return 0;
  }
  MemMove(spiel, pos, sizeof(struct spiel));
  return pos;
}

void GetMatch(UInt8 num, struct spiel* spiel)
{
  char *p = MemHandleLock(DmQueryRecord(worldcupDB,1));
  MemMove(spiel, p+sizeof(struct spiel)*num, sizeof(struct spiel));
  MemPtrUnlock(p);
  return;
}

void WriteFixture(struct match *match, UInt8 num, UInt8 group)
{
  struct spiel Spiel;
  int i;
  char *p = MemHandleLock(DmGetRecord(worldcupDB,1));

  for (i=0; i<num; i++) {
    Spiel.gruppe = group;
    Spiel.datum = match[i].datum;
    Spiel.ort = match[i].ort;
    Spiel.e1 = match[i].erg1;
    Spiel.e2 = match[i].erg2;
    Spiel.m1 = match[i].team1;
    Spiel.m2 = match[i].team2;
    DmWrite(p, match[i].recNo*sizeof(struct spiel), 
	    &Spiel, sizeof(struct spiel));
  }
  MemPtrUnlock(p);
  DmReleaseRecord(worldcupDB,1,true);
}

void SetHandleString(FieldType *fptr, int num)
{
  MemHandle handle;
  char *p;
  struct team *team;

  if ( (handle = DmQueryRecord(worldcupDB,3)) ) {
    p = MemHandleLock(handle);
    MemHandleUnlock(handle);
    p += num*sizeof(struct team);
    team = (struct team*) p;
    SetTextFieldWithString(fptr, team->name, false);
  }

}

struct team* GetTeamString(int num)
{
  MemHandle handle;
  char *p;
  struct team *team;

  if ( num && (handle = DmQueryRecord(worldcupDB,3)) ) {
    p = MemHandleLock(handle);
    p += (num-1)*sizeof(struct team);
    team = (struct team*) p;
    MemHandleUnlock(handle);
    return team;
  }
  return 0;
}

UInt8 ComputeSetHandleString(FieldType *fptr, int num)
{
  char *p, buffer[20];
  struct match match[6];
  struct mstatus table[4];
  struct spiel Spiel;
  int platz, gruppe, final, i, rec, newnum;

  if (num>10 && num<30) {
    platz = num / 10;
    gruppe = num % 10;
    ReadSetFixture(match,gruppe,false);
    final = ComputeTable(table,match,15,0);
    if (!final) {
      if (fptr) {
	p = GetString(IdDefaultName);
	StrIToA(buffer,platz);
	StrCat(buffer,". ");
	StrCat(buffer,p);
	StrCat(buffer,"  ");
	buffer[StrLen(buffer)-1] = 'A'-1+gruppe;
	SetTextFieldWithString(fptr,buffer,false);
      }
      // indicates that the finalalists are yet unknown
      return num+64;
    }
    for (i=0; i<3; i++) {
      if (table[i].Platz == platz)
	break;
    }
    if (fptr)
      SetHandleString(fptr, (gruppe-1)*4+i);
    return (gruppe-1)*4+i+1;
  } 

  if (num>48 && num<165) {
    platz = num / 100 + 1;
    rec   = num % 100 - 1;
    GetMatch(rec,&Spiel);
    newnum = num;
    if (platz == 1) {
      if (Spiel.e1 > Spiel.e2) {
	newnum = Spiel.m1;
      }
      if (Spiel.e1 < Spiel.e2) {
	newnum = Spiel.m2;
      }
    }
    if (platz == 2) {
      if (Spiel.e1 > Spiel.e2) {
	newnum = Spiel.m2;
      }
      if (Spiel.e1 < Spiel.e2) {
	newnum = Spiel.m1;
      }
    }
    if (newnum<num) 
      return ComputeSetHandleString(fptr,newnum);
    else {
      if (fptr) {
	p = GetString(IdDefaultName+1);
	StrIToA(buffer,platz);
	StrCat(buffer,". ");
	StrCat(buffer,p);
	StrCat(buffer," ");
	StrIToA(buffer+StrLen(buffer),rec+1);
	SetTextFieldWithString(fptr,buffer,false);
      }
    }
  }
  return rec+64;
}

UInt32 SetOrtString(Int16 id, int num)
{
  MemHandle handle;
  char *p, *label;
  struct ort *ort;
  ControlType *ctlP = Id2Ptr(id);
  label = (char*)CtlGetLabel(ctlP);

  if ( (handle = DmQueryRecord(worldcupDB,2)) ) {
    p = MemHandleLock(handle);
    p += num*sizeof(struct ort);
    ort = (struct ort*) p;
    CtlEraseControl(ctlP);
    StrCopy(label,ort->name);
    CtlDrawControl(ctlP);
    MemHandleUnlock(handle);
    return ort->cap;
  }
  return 0;
}

UInt16 readPlayers(char *items[], UInt16 selectedTeam)
{
  UInt16 i = 0;
  MemHandle handle;
  char *p, *end;

  if ( (handle = DmQueryRecord(worldcupDB,3+selectedTeam)) ) {
    p = MemHandleLock(handle);
    end = p+MemHandleSize(handle);
    while (p<end) {
      items[i++] = p;
      p += sizeof(struct player);
    }
    MemHandleUnlock(handle);
  }
  return i;
}

