/*
 * Worldcup is Copyright (c) 2001,2002 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "worldcup.h"
#include "worldcuprcp.h"

#include "datedb.h"

static UInt32 fDatum[100];
static TablePtr matchesTable;

/**
 * Matches Form
 */
void RedrawTable(TablePtr tblP, UInt16 first,
		 Int8 rows, UInt16 up, UInt16 down)
{
  UInt16 row, last = first+9;
  ControlType *upP = Id2Ptr(up);
  ControlType *downP = Id2Ptr(down);
  char *label;

  TblUnhighlightSelection(tblP);
  if (last>rows-1) last=rows-1;
  for (row=0; row<TblGetNumberOfRows(tblP); row++)
    TblSetRowUsable(tblP, row, 
		    (row>=first && row<=last) ? true : false);

  label = (char*)CtlGetLabel(upP);
  label[0] = (first==0) ? 3 : 1;
  label = (char*)CtlGetLabel(downP);
  label[0] = (last==rows-1) ? 4 : 2;
  
  CtlDrawControl(upP);
  CtlDrawControl(downP);

  TblMarkTableInvalid(tblP);
  TblRedrawTable(tblP);
}

static void SetMatchesListTeamString(Int16 num, char *p, char *buf)
{
  struct team *team;
  Int16 platz, gruppe;
  if ( num > 0) {
    if ( num <= 32 ) {
      p += (num-1)*sizeof(struct team);
      team = (struct team*) p;
      StrCopy(buf,team->name);
    } else {
      platz = (num-64)/10;
      gruppe = (num-64)%10;
      p = GetString(IdDefaultName);
      StrIToA(buf,platz);
      StrCat(buf,". Gr ");
      // StrCat(buf,p);
      StrCat(buf,"  ");
      buf[StrLen(buf)-1] = 'A'-1+gruppe;
    }
  }
}

static void DrawMatchesItem(void* tableP, Int16 row, Int16 column, 
                            RectangleType *bounds)
{
  Int16 n, num;
  UInt16 x;
  Char buf[64], *p, *q;
  DateTimeType dateP;
  MemHandle handle;

  n = TblGetItemInt(tableP, row, column);
  x = bounds->topLeft.x;

  FntSetFont(0);

  switch (column) {
  case 0:
    TimSecondsToDateTime(n<64?(fDatum[n]+appPrefs.gmtOff):0, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
		PrefGetPreference(prefDateFormat), buf);
    break;

  case 1:
    TimSecondsToDateTime(n<64?(fDatum[n]+appPrefs.gmtOff):0, &dateP);
    TimeToAscii(dateP.hour,dateP.minute,
		tfColon24h, buf);
    x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    break;

  case 2:
    if ( (handle = DmQueryRecord(worldcupDB,3)) ) {
      p = q = MemHandleLock(handle);
      MemHandleUnlock(handle);

      if ( (num = (n%256)) < 112) {
	SetMatchesListTeamString(n>>8,p,buf);      
	StrCat(buf,"-");
	SetMatchesListTeamString(num,q,buf+StrLen(buf));
      } else {
        if (num<120) {
	  StrCopy(buf,GetString(IdRoundString+2));
	} else if (num<125) {
	  StrCopy(buf,GetString(IdRoundString+3));
	} else {
	  StrCopy(buf,GetString(IdRoundString+4));
	}
      }

    }
    break;

  case 3:
    if (n == -1) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n);
    }
    StrCat(buf," :");
    x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    break;

  case 4:
    if (n == -1) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n);
    }
    break;
  }

  WinSetClip(bounds);
  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
  WinResetClip();
}  

static void MakeDatebookEntry(UInt8 row)
{
  UInt32 secs = fDatum[row]+appPrefs.gmtOff;
  ApptDBRecordType dbrec;
  ApptDateTimeType dbwhen;
  AlarmInfoType dbalarm;
  Int16 num, n = TblGetItemInt(matchesTable, row, 2);
  MemHandle handle;
  DateTimeType date_time;
  UInt16 index;

  MemSet(&dbrec, sizeof(ApptDBRecordType), 0);
  MemSet(&dbwhen, sizeof(ApptDateTimeType), 0);
  MemSet(&dbalarm, sizeof(AlarmInfoType), 0);

  dbrec.when = &dbwhen;
  dbrec.alarm = &dbalarm;
  dbrec.repeat = NULL;
  dbrec.note = NULL;
  dbrec.description = NULL;

  if ( (handle = DmQueryRecord(worldcupDB,3)) ) {
    Char buf[64], *p, *q;
    p = q = MemHandleLock(handle);
    MemHandleUnlock(handle);
    
    if ( (num = (n%256)) < 112) {
      SetMatchesListTeamString(n>>8,p,buf);      
      StrCat(buf,"-");
      SetMatchesListTeamString(num,q,buf+StrLen(buf));
    } else {
      if (num<120) {
	StrCopy(buf,GetString(IdRoundString+2));
      } else if (num<125) {
	StrCopy(buf,GetString(IdRoundString+3));
      } else {
	StrCopy(buf,GetString(IdRoundString+4));
      }
    }
    dbrec.description = buf;
  }
  
  DateSecondsToDate(secs,&dbwhen.date);
  TimSecondsToDateTime(secs, &date_time);
  if (secs % 60)
    TimAdjust(&date_time, (60 - (secs % 60)));
  dbwhen.startTime.hours = date_time.hour;
  dbwhen.startTime.minutes = date_time.minute;
  if (dbwhen.startTime.hours < 22) {
    dbwhen.endTime.hours = dbwhen.startTime.hours+2;
    dbwhen.endTime.minutes = dbwhen.startTime.minutes;
  } else {
    dbwhen.endTime.hours = 23;
    dbwhen.endTime.minutes = 59;
  }

  dbalarm.advanceUnit = aauMinutes;
  dbalarm.advance = 0;

  ApptNewRecord(datebook, &dbrec, &index);
}

static void InitFormMatches(void)
{
  Int16 col, row;
  char *p;
  struct spiel Spiel;

  matchesTable = Id2Ptr(IdFrmMatches);

  for (row=0; row<TblGetNumberOfRows(matchesTable); row++) {
    TblSetRowSelectable(matchesTable, row, true);
    TblSetRowUsable(matchesTable, row, row<10?true:false);
    for (col=0; col<=4; col++) {
      TblSetItemStyle(matchesTable, row, col, customTableItem);
      TblSetColumnUsable(matchesTable, col, true);
      TblSetCustomDrawProcedure(matchesTable, col, DrawMatchesItem);
    }
  }

  for ( p=ReadFixture(0,&Spiel),row=0; p; p=ReadFixture(p,&Spiel),row++ ) {
    if (row>63) continue;
    TblSetItemInt(matchesTable, row, 0, row);
    TblSetItemInt(matchesTable, row, 1, row);
    fDatum[row] = Spiel.datum;
    if (Spiel.gruppe>0)
      TblSetItemInt(matchesTable, row, 2, 
		    ((Spiel.m1+(Spiel.gruppe-1)*4)<<8)
		    + Spiel.m2+(Spiel.gruppe-1)*4);
    else
      TblSetItemInt(matchesTable, row, 2, 
		    (ComputeSetHandleString(0, Spiel.m1)<<8)
		    + ComputeSetHandleString(0, Spiel.m2));
    TblSetItemInt(matchesTable, row, 3, Spiel.e1);
    TblSetItemInt(matchesTable, row, 4, Spiel.e2);
  }

  FrmDrawForm(FrmGetActiveForm());
  // TblDrawTable(matchesTable);
}

Boolean MatchesFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static Int8 row = 0;
  Int8 oldRow;

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFormMatches();
    handled = true; 
    break;

  case menuEvent:

    switch(e->data.menu.itemID) {

    case IdMenuItem:
      FrmPopupForm(IdFrmAbout);
      break;

    case IdMenuItem+1:
      FrmHelp(IdFrmMatches);
      break;

    case IdMenuItem+2:
      FrmPopupForm(IdFrmTimezone);
      break;

    case IdMenuItem+3:
      FrmReturnToForm(0);
      break;

    }
    handled = true;
    break;

  case tblSelectEvent:
    if ( 0 == DisplayConfirmationMessage(IdErrorMsg+4)) {
      if (OpenDatebook()) {
	MakeDatebookEntry(e->data.tblSelect.row);
	DmCloseDatabase(datebook);
      }
    }
    TblUnhighlightSelection(matchesTable);
    handled = true;
    break;

  case ctlSelectEvent:
    oldRow = row;
    if (e->data.ctlSelect.controlID == IdFrmMatches+1) {
      row -= 10;
      if (row<0) row = 0;
    }
    if (e->data.ctlSelect.controlID == IdFrmMatches+2) {
      row += 10;
      if (row>54) row = 54;
    }
    if (oldRow != row)
      RedrawTable(matchesTable, row, 64, IdFrmMatches+1, IdFrmMatches+2);
    handled = true; 
    break;

  default:
    break;
  }

  return handled;
}

