/*
 * Worldcup is Copyright (c) 2001,2002 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "worldcup.h"
#include "worldcuprcp.h"

void* Id2Ptr(UInt16 id)
{ 
  FormPtr frm = FrmGetActiveForm();
  return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,id));
}

char* GetString(UInt16 id)
{
  char *p;
  MemHandle h = DmGetResource('tSTR', id);
  if (!h) return NULL;
  p = MemHandleLock(h);
  MemHandleUnlock(h);
  return p;
}

void DisplayErrorMessage(UInt16 id)
{
  char *msg = GetString(id);
  FrmCustomAlert(IdAlertError,msg,NULL,NULL);
}

void DisplayInformationMessage(UInt16 id)
{
  char *msg = GetString(id);
  FrmCustomAlert(IdAlertInformation,msg,NULL,NULL);
}

UInt16 DisplayConfirmationMessage(UInt16 id)
{
  char *msg = GetString(id);
  return FrmCustomAlert(IdAlertConfirmation,msg,NULL,NULL);
}

void SetHandleInt(FieldType* fptr, Int32 n)
{
  MemHandle h;
  if (!(h = FldGetTextHandle(fptr))) {
    h = MemHandleNew(7);
  }
  if (n==-1)
    StrCopy(MemHandleLock(h),"");
  else
    StrIToA(MemHandleLock(h), n);
  MemHandleUnlock(h);
  FldSetTextHandle(fptr, h);
}

void SetTextFieldWithHandle(FieldPtr textFldP, MemHandle textH, Boolean drawIt)
{
  MemHandle oldTextH =  FldGetTextHandle(textFldP);
  FldSetTextHandle(textFldP,textH);
  if (drawIt)
    FldDrawField(textFldP);
  if (oldTextH)
    MemHandleFree(oldTextH);
}

Err SetTextFieldWithString(FieldPtr textFldP, char *string, Boolean drawIt)
{
  Err anErr = errNone;
  MemHandle textH = MemHandleNew(StrLen(string)+sizeof(Char));
  if (textH) {
    StrCopy(MemHandleLock(textH),string);
    MemHandleUnlock(textH);
    SetTextFieldWithHandle(textFldP,textH,drawIt);
  } else {
    anErr = memErrNotEnoughSpace;
  }
  return anErr;
}

